<?php

namespace Sugarcrm\Sugarcrm\custom\PxRoutines\LBExtensions\Templates\Helpers\TemplateEngines;

require_once 'include/SugarSmarty/Sugar_Smarty.php';

class SmartyTemplateEngine extends \Sugar_Smarty{
    use TemplateEngineTrait;

    public $compiler_file = 'custom/PxRoutines/LBExtensions/Templates/Helpers/TemplateEngines/Compilers/LBTemplatesSmartyCompiler.php';
    public $compiler_class = 'LBTemplatesSmartyCompiler';

    public function fetchLBTemplate(){
        return parent::fetch($this->getResourceName());
    }

    public function assignLBVariable($variable, $value){
        parent::assign($variable, $value);
    }

    function _fetch_resource_info(&$params)
    {
        if(!empty($this->lbTemplateBean)){
            $params['source_content'] = $this->getSourceContent();
            $params['resource_timestamp'] = $this->getResourceTimestamp();
            return true;
        }
        return parent::_fetch_resource_info($params);
    }

    function trigger_error($error_msg, $error_type = E_USER_WARNING)
    {
        throw new \Exception("Smarty $error_msg", $error_type);
    }

    private function getResourceName(){
        if(empty($this->lbTemplateBean) || empty($this->lbTemplateBean->id))
            return 'New LB Template';

        global $sugar_config;
        $parsedSiteUrl = parse_url($sugar_config['site_url']);
        $host = $parsedSiteUrl['host'];
        if (!isset($parsedSiteUrl['port'])) {
            $parsedSiteUrl['port'] = 80;
        }
        $port = ($parsedSiteUrl['port'] != 80) ? ":" . $parsedSiteUrl['port'] : '';
        $path = !empty($parsedSiteUrl['path']) ? $parsedSiteUrl['path'] : "";
        $cleanUrl = "{$parsedSiteUrl['scheme']}://{$host}{$port}{$path}";

        return $cleanUrl . '/#PX_FlowchartTemplates/'.$this->lbTemplateBean->id;
    }

    private function getSourceContent(){
        if(empty($this->lbTemplateBean))
            return '';

        $text = $this->lbTemplateBean->getHtmlBody();
        $smartyExpressions = array();
        preg_match_all('/{(.*?)}/', $text, $smartyExpressions);

        if(!empty($smartyExpressions[0])){
            $inLiteralExpression = 0;
            foreach($smartyExpressions[0] as $expression){
                if($expression == '{literal}'){
                    $inLiteralExpression ++;
                }elseif($expression == '{/literal}'){
                    $inLiteralExpression --;
                }elseif(!$inLiteralExpression){
                    $newExpression = html_entity_decode(strip_tags($expression));
                    $text = str_replace($expression, $newExpression, $text);
                }
            }
        }

        return $text;
    }

    private function getResourceTimestamp(){
        if(empty($this->lbTemplateBean) || empty($this->lbTemplateBean->date_modified))
            return time();

        global $timedate, $current_user;
        $date = $this->lbTemplateBean->date_modified;

        $sugarDT = $timedate->fromUserType($date, 'datetime', $current_user);
        if(empty($sugarDT))
            $sugarDT = $timedate->fromDbType($date, 'datetime');

        return empty($sugarDT) ? time() : $sugarDT->getTimestamp();
    }

}


class SmartyTemplateException extends \Exception{

    public function setFile($file){
        $this->file = $file;
        return $this;
    }

    public function setLine($line){
        $this->file = $line;
        return $this;
    }
}