({
    extendsFrom: 'PreviewView',

    saveClicked: function() {
        if(!_.isUndefined(this.getField('context')))
            this.getField('context').beforeHandleSave();

        this.checkContextFieldLength();
    },

    checkContextFieldLength: function(){
        if(!this.getField('context').isFieldLengthValid()){

            var symbolsAmountViolation = this.getField('context').getSymbolsAmountViolation(),
                errorMsg = app.lang.get('LBL_CONTEXT_MAX_LENGTH_EXCEEDED', 'PX_FlowchartTemplates').replace('%d', symbolsAmountViolation);

            app.alert.show('lb_tpl_context_max_length_exceeded', {
                level: 'confirmation',
                messages: errorMsg,
                onConfirm: _.bind(function(){
                    this.getField('context').substringContextFieldLength();
                    this.checkTemplateParseErrors();
                }, this)
            });
        }else{
            this.checkTemplateParseErrors();
        }
    },

    checkTemplateParseErrors: function(){
        var self = this,
            url = app.api.buildURL('lbTemplate/validateTemplate', '', {}, {}),
            engineTypesList = app.lang.getAppListStrings('px_lbtemplates_tpl_engine_type'),
            engineTypeName = self.model.get('tpl_engine_type'),
            engineTypeLabel = engineTypesList[engineTypeName] || '',
            errorMsg = app.lang
                .get('MSG_TEMPLATE_SYNTAX_ERROR_TEXT', self.module)
                .replace('$TPL_ENGINE$', engineTypeLabel);

        app.alert.show('validation-lb-templates', {level: 'process'});
        app.api.call('create', url,
            {
                name: this.model.get('name'),
                tpl_engine_type: this.model.get('tpl_engine_type'),
                context: this.model.get('context'),
                html_body: this.model.get('html_body'),
            },
            {
                success: _.bind(function (response) {
                    app.alert.dismiss('validation-lb-templates');
                    app.alert.dismiss('lb-templates-parse-error');
                    if(response.errorCode){
                        errorMsg =  errorMsg + '<br>' + response.errorText;
                        // errorMsg =  errorMsg + '<br>' + app.lang.get('MSG_TEMPLATE_SYNTAX_ERROR_WARNING', self.module);
                        app.alert.show('lb_template_err_variable', {
                            level: 'confirmation',
                            messages: errorMsg,
                            onConfirm: function(){
                                self._super("saveClicked");
                            }
                        });
                    }else{
                        self._super("saveClicked");
                    }
                    return;
                }),
                error: _.bind(function () {
                    app.alert.dismiss('validation-lb-templates');
                    app.alert.show('lb_template_err_variable', {
                        level: 'confirmation',
                        messages: errorMsg,
                        onConfirm: function(){
                            self._super("saveClicked");
                        }
                    });
                    return;
                })
            });
    },

    hasUnsavedChanges: function() {
        var changedAttributes,
            editableFieldNames = [],
            unsavedFields,
            self = this,
            setAsEditable = function(fieldName) {
                if (fieldName && _.indexOf(self.noEditFields, fieldName) === -1) {
                    editableFieldNames.push(fieldName);
                }
            };

        if (this.resavingAfterMetadataSync)
            return false;

        var syncedAttributes = this.model.getSynced();

        if(!_.isUndefined(this.getField('html_body')) && syncedAttributes['html_body'] != this.model.get('html_body')) {
            var syncedHtmlBody = this.getField('html_body').syncedValue;
            syncedAttributes['html_body'] = syncedHtmlBody;
        }

        // changedAttributes = this.model.changedAttributes(this.model.getSynced());
        changedAttributes = this.model.changedAttributes(syncedAttributes);

        if(_.isEmpty(changedAttributes)){
            if(this.getField('context')){
                var syncedAttributes = this.model.getSynced();
                var argumentFields = this.getField('context').getFieldsMeta();
                _.each(argumentFields, function(value, key){
                    if((_.isUndefined(syncedAttributes[value.name]) && !_.isUndefined(this.model.get(value.name))) || syncedAttributes[value.name] != this.model.get(value.name)){
                        changedAttributes = changedAttributes || {};
                        changedAttributes[value.name] = syncedAttributes[value.name] || '';
                    }
                }, this);
            }
        }

        if (_.isEmpty(changedAttributes)) {
            return false;
        }

        // get names of all editable fields on the page including fields in a fieldset
        _.each(this.meta.panels, function(panel) {
            _.each(panel.fields, function(field) {
                if (!field.readonly) {
                    setAsEditable(field.name);

                    if(field.name == 'context'){
                        field.fields = this.getField('context').getFieldsMeta();
                    }

                    if (field.fields && _.isArray(field.fields)) {
                        _.each(field.fields, function(field) {
                            setAsEditable(field.name);
                        });
                    }
                }
            }, this);
        }, this);

        // check whether the changed attributes are among the editable fields
        unsavedFields = _.intersection(_.keys(changedAttributes), editableFieldNames);

        return !_.isEmpty(unsavedFields);
    },
})
